package Text::BibTeX::Section;
use strict;
use Carp;
our $AUTOLOAD;

my %fields=(
	    ANNOTE=>undef,
	    KEY=>undef,
	    STYLE=>undef,
	    ENTRIES=>undef,
	    INDEX=>undef,
	    TITLE=>undef,
	    WIDESTLABEL=>undef,
	    SORTDIR=>undef,
	    LEVEL=>undef,
            SELECT=>undef,
	    STRUCTURE=>undef,
	    COUNT=>undef,
	    OMITEMPTY=>undef,
	    OPTIONS=>{},
	   );

my %defaultwidest=(
		plain => '[99.99]',
		dated => '2004'
);

sub new{
  my $class = shift;
  my %opts=@_;

  my $self  = {_permitted=>\%fields};
  foreach my $key (keys %opts){
    (exists $fields{$key}) or die ("Unknown parameter $key");
    $self->{$key}=$opts{$key};
  }
  $self->{STYLE}   = $self->{STYLE} || "plain";
  $self->{ENTRIES} =  [];
  $self->{WIDESTLABEL}= $self->{WIDESTLABEL} || $defaultwidest{$self->{STYLE}};
  $self->{SORTDIR}=$self->{SORTDIR} || 1;
  $self->{LEVEL}=$self->{LEVEL} || 1;
  $self->{COUNT}=0;
  $self->{OMITEMPTY} = $self->{OMITEMPTY} || 0;
  bless ($self, $class);
  return $self;
};

sub add_entry($$){
  my $self=shift;
  my $entry=shift;
  

  push (@{$self->{ENTRIES}},$entry);
  $self->{COUNT}++
}

my %collectmap=();

sub collected($$){
  
  my $self=shift;

  my $entry=shift;
  my $key=$entry->key();
  my $type=$entry->type;


  my $keyref=$self->get_option('collect') ||
    $entry->get_option('collect');

  return 0 if (!defined($keyref));

  my $collectkey=join(",",$type,map($entry->get($_), @{$keyref}));
  if (!defined($collectmap{$collectkey})){
    $entry->set("repcount",1);
    $entry->{COLLECTHASH}={};
    $collectmap{$collectkey}=$entry;
    return 0;
  } else {
    my $old=$collectmap{$collectkey};
    my $repcount=$old->get("repcount");
    my $year=List::Util::min($entry->get("year"),$old->get("year"));
    my $endyear=List::Util::max($entry->get("endyear") || $entry->get("year"),
				$old->get("endyear") || $old->get("year") );
    $old->set("year",$year);
    $old->set("endyear",$endyear);

    $repcount++;
    $old->set("repcount",$repcount);
    if (($entry->type eq 'refreport') || ($entry->type eq 'talk')){
      my $specialfield;
      if ($entry->type eq 'refreport'){
	$specialfield='journal';
      } else {
	$specialfield='institution';
      }


      my $specialval=$entry->get($specialfield);
      my $hashref=$old->{COLLECTHASH};
 #     print STDERR "checking for <$specialval>\n";
      
 #     print STDERR $entry->key();
      if (!$hashref->{$specialval}){
	$hashref->{$specialval}=1;
      }
      
      my $string=join(", ", 
		     sort (keys %{$hashref}));
#      print STDERR "updating to ",$string,"\n";
      $old->set($specialfield,$string);
    }
      
    return 1;
  }
		      
  return 0;
}


sub get_option($$){
  my $self=shift;
  my $key=shift;
  
  if (defined($self->{OPTIONS}) && exists($self->{OPTIONS}{$key})){
    return $self->{OPTIONS}{$key};
  } else {
    return undef;
  }
}

sub sort($){
  my $self=shift;

  my @entries=@{$self->entries};

#  print STDERR "Sorting";

  @entries=  sort { $self->{SORTDIR} * ($a->sort_key cmp $b->sort_key) } @entries;

  
  $self->{ENTRIES}=[@entries];
}


sub structure { shift->{STRUCTURE} }

sub options { shift->{OPTIONS} }

sub AUTOLOAD {
  my $self = shift;
  my $type = ref($self)
    or croak "$self is not an object";
  
  my $name = $AUTOLOAD;


  $name =~ s/.*://;   # strip fully-qualified portion

  return unless $name =~ /[^A-Z]/;  # skip DESTROY and all-cap methods

  
  $name=uc($name);

  unless (exists $self->{_permitted}->{$name} ) {
    croak "Can't access `$name' field in class $type";
  }

  if (@_) {
    return $self->{$name} = shift;
  } else {
    return $self->{$name};
  }
}

sub markup($$){
  my $self=shift;
  my $key=shift;

  return $self->structure->get_options($key.'_mkup');

}
1;
